;; $Id: dbautoc.dsl 0.1 1997/07/31 14:51:33 nwalsh Exp $

;; ========================== TABLE OF CONTENTS =========================

(define (NUMLABEL fmt)
  (let ((div0n 
	 ;; Enumerate Chapters and Appendixes
	 (cond
	  ((have-ancestor? "CHAPTER") (ancestor-child-number "CHAPTER"))
	  ((have-ancestor? "APPENDIX") (ancestor-child-number "APPENDIX"))
	  (else #f)))
	(div1n (ancestor-child-number "SECT1"))
	(div2n (ancestor-child-number "SECT2"))
	(div3n (ancestor-child-number "SECT3"))
	(div4n (ancestor-child-number "SECT4"))
	(div5n (ancestor-child-number "SECT5")))
    (string-append
     (if div0n (string-append (NESTEDFNUM div0n fmt)) " ")
     (if div1n (string-append "." (NESTEDFNUM div1n "1"))
	 (if div0n ". " " "))
     (if div2n (string-append "." (NESTEDFNUM div2n "1")) " ")
     (if div3n (string-append "." (NESTEDFNUM div3n "1")) " ")
     (if div4n (string-append "." (NESTEDFNUM div4n "1")) " ")
     (if div5n (string-append "." (NESTEDFNUM div5n "1")) " "))))

(mode toc
      (element (PART TITLE) ($toc-entry$ 1))
      (element (PREFACE TITLE) ($toc-entry$ 1))
      (element (CHAPTER TITLE) ($toc-entry$ 1))
      (element (APPENDIX TITLE) ($toc-entry$ 1))
      (element (GLOSSARY TITLE) ($toc-entry$ 1))
      (element (BIBLIOGRAPHY TITLE) ($toc-entry$ 1))
      (element (INDEX TITLE) ($toc-entry$ 1))
      (element (SECT1 TITLE) ($toc-entry$ 2))
      (element (SECT2 TITLE) ($toc-entry$ 3))
      (element (SECT3 TITLE) ($toc-entry$ 4))
      (element (SECT4 TITLE) ($toc-entry$ 5))
      (element (SECT5 TITLE) ($toc-entry$ 6))
      (element (REFERENCE TITLE) ($toc-entry$ 1))
      (default (apply process-matching-children
		      (list "PART" "PREFACE" "CHAPTER" "APPENDIX" 
                            "GLOSSARY" "BIBLIOGRAPHY" "INDEX"
                            "SECT1" "SECT2" "SECT3" "SECT4" "SECT5" 
			    "REFERENCE" 
			    "TITLE"))))

(define %toc-indent% 2pi)
(define %toc-spacing-factor% 0.4)

(define ($toc-entry$ level)
  (let* ((pelem (parent))
	 (href (href-to pelem)))
    (sosofo-append
     (make sequence
       (literal
	(cond
	 ((have-ancestor? "CHAPTER") (NUMLABEL "1"))
	 ((have-ancestor? "APPENDIX") (NUMLABEL "A"))
	 (else "")))
       (make element gi: "A"
	     attributes: (list
			  (list "HREF" href))
	     (with-mode #f (process-children-trim)))
       (make empty-element gi: "BR")))))

(define (MAKETOC)
  (sosofo-append
   (make element gi: "DIV"
	 attributes: (list
		      (list "CLASS" "TOC"))
	 (make element gi: "P"
	       (make element gi: "B"
		     (literal (gentext-element-name "TOC")))
	       (make empty-element gi: "BR")
	       (with-mode toc
		 ;; Handle two case: books with and without parts.
		 ;; Books with Parts may begin with Preface(s) and are
		 ;; followed by Parts.  Books without parts are just
		 ;; sequential collections of chapter-level
		 ;; components.  If you have something more complex,
		 ;; you'll have to tinker here...
		 (let ((book (if (equal? (gi) "BOOK")
				 (current-node)
				 (ancestor "BOOK"))))
		   (if (node-list-empty?
			(select-elements (descendants book) "PART"))
		       
		       (sosofo-append
			(process-node-list
			 (select-elements (descendants book) "PREFACE"))
			(process-node-list
			 (select-elements (descendants book) "CHAPTER"))
			(process-node-list
			 (select-elements (descendants book) "REFERENCE"))
			(process-node-list
			 (select-elements (descendants book) "APPENDIX"))
			(process-node-list
			 (select-elements (descendants book) "GLOSSARY"))
			(process-node-list
			 (select-elements (descendants book) "BIBLIOGRAPHY"))
			(process-node-list
			 (select-elements (descendants book) "INDEX")))
		       
		       (sosofo-append
			(process-node-list
			 (select-elements (descendants book) "PREFACE"))
			(process-node-list
			 (select-elements (descendants book) "PART"))
			(process-node-list
			 (select-elements (descendants book) "APPENDIX"))
			(process-node-list
			 (select-elements (descendants book) "GLOSSARY"))
			(process-node-list
			 (select-elements (descendants book) "BIBLIOGRAPHY"))
			(process-node-list
			 (select-elements (descendants book) "INDEX")))
		       )))))
	 ))
